@@ -13,6 +13,8 @@
 
 #include <config.h>
 
+#include <stdbool.h>
+
 #include <isc/magic.h>
 #include <isc/mem.h>
 #include <isc/types.h>
@@ -47,7 +49,6 @@ struct dns_order {
 isc_result_t
 dns_order_create(isc_mem_t *mctx, dns_order_t **orderp) {
 	dns_order_t *order;
-	isc_result_t result;
 
 	REQUIRE(orderp != NULL && *orderp == NULL);
 
@@ -58,11 +59,7 @@ dns_order_create(isc_mem_t *mctx, dns_order_t **orderp) {
 	ISC_LIST_INIT(order->ents);
 
 	/* Implicit attach. */
-	result = isc_refcount_init(&order->references, 1);
-	if (result != ISC_R_SUCCESS) {
-		isc_mem_put(mctx, order, sizeof(*order));
-		return (result);
-	}
+	isc_refcount_init(&order->references, 1);
 
 	order->mctx = NULL;
 	isc_mem_attach(mctx, &order->mctx);
@@ -98,7 +95,7 @@ dns_order_add(dns_order_t *order, const dns_name_t *name,
 	return (ISC_R_SUCCESS);
 }
 
-static inline isc_boolean_t
+static inline bool
 match(const dns_name_t *name1, const dns_name_t *name2) {
 
 	if (dns_name_iswildcard(name2))
@@ -131,29 +128,25 @@ void
 dns_order_attach(dns_order_t *source, dns_order_t **target) {
 	REQUIRE(DNS_ORDER_VALID(source));
 	REQUIRE(target != NULL && *target == NULL);
-	isc_refcount_increment(&source->references, NULL);
+	isc_refcount_increment(&source->references);
 	*target = source;
 }
 
 void
 dns_order_detach(dns_order_t **orderp) {
+	REQUIRE(orderp != NULL && DNS_ORDER_VALID(*orderp));
 	dns_order_t *order;
-	dns_order_ent_t *ent;
-	unsigned int references;
-
-	REQUIRE(orderp != NULL);
 	order = *orderp;
-	REQUIRE(DNS_ORDER_VALID(order));
-	isc_refcount_decrement(&order->references, &references);
 	*orderp = NULL;
-	if (references != 0)
-		return;
 
-	order->magic = 0;
-	while ((ent = ISC_LIST_HEAD(order->ents)) != NULL) {
-		ISC_LIST_UNLINK(order->ents, ent, link);
-		isc_mem_put(order->mctx, ent, sizeof(*ent));
+	if (isc_refcount_decrement(&order->references) == 1) {
+		isc_refcount_destroy(&order->references);
+		order->magic = 0;
+		dns_order_ent_t *ent;
+		while ((ent = ISC_LIST_HEAD(order->ents)) != NULL) {
+			ISC_LIST_UNLINK(order->ents, ent, link);
+			isc_mem_put(order->mctx, ent, sizeof(*ent));
+		}
+		isc_mem_putanddetach(&order->mctx, order, sizeof(*order));
 	}
-	isc_refcount_destroy(&order->references);
-	isc_mem_putanddetach(&order->mctx, order, sizeof(*order));
 }
