@@ -168,13 +168,13 @@ compress_test(dns_name_t *name1, dns_name_t *name2, dns_name_t *name3,
 	isc_buffer_setactive(&source, source.used);
 
 	dns_name_init(&name, NULL);
-	RUNTIME_CHECK(dns_name_fromwire(&name, &source, dctx, false,
+	RUNTIME_CHECK(dns_name_fromwire(&name, &source, dctx, 0,
 					&target) == ISC_R_SUCCESS);
-	RUNTIME_CHECK(dns_name_fromwire(&name, &source, dctx, false,
+	RUNTIME_CHECK(dns_name_fromwire(&name, &source, dctx, 0,
 					&target) == ISC_R_SUCCESS);
-	RUNTIME_CHECK(dns_name_fromwire(&name, &source, dctx, false,
+	RUNTIME_CHECK(dns_name_fromwire(&name, &source, dctx, 0,
 					&target) == ISC_R_SUCCESS);
-	RUNTIME_CHECK(dns_name_fromwire(&name, &source, dctx, false,
+	RUNTIME_CHECK(dns_name_fromwire(&name, &source, dctx, 0,
 					&target) == ISC_R_SUCCESS);
 	dns_decompress_invalidate(dctx);
 
@@ -606,7 +606,6 @@ getlabel_test(void **state) {
 		dns_fixedname_t f1, f2;
 		dns_name_t *n1, *n2;
 		dns_label_t l1, l2;
-		unsigned char *p1, *p2;
 		unsigned int j;
 
 		n1 = dns_fixedname_initname(&f1);
@@ -623,10 +622,8 @@ getlabel_test(void **state) {
 		dns_name_getlabel(n2, testcases[i].pos2, &l2);
 		assert_int_equal(l1.length, l2.length);
 
-		p1 = l1.base;
-		p2 = l2.base;
 		for (j = 0; j < l1.length; j++) {
-			assert_int_equal(*p1++, *p2++);
+			assert_int_equal(l1.base[j], l2.base[j]);
 		}
 	}
 }
