@@ -12,6 +12,8 @@
 #ifndef ISC_UTIL_H
 #define ISC_UTIL_H 1
 
+#include <inttypes.h>
+
 /*! \file isc/util.h
  * NOTE:
  *
@@ -217,6 +219,10 @@ extern void mock_assert(const int result, const char* const expression,
 	(mock_assert(0, #expression, __FILE__, __LINE__), abort()) : (void)0)
 #define _assert_true(c, e, f, l) \
 	((c) ? (void)0 : (_assert_true(0, e, f, l), abort()))
+#define _assert_int_equal(a, b, f, l) \
+	(((a) == (b)) ? (void)0 : (_assert_int_equal(a, b, f, l), abort()))
+#define _assert_int_not_equal(a, b, f, l) \
+	(((a) != (b)) ? (void)0 : (_assert_int_not_equal(a, b, f, l), abort()))
 #else /* UNIT_TESTING */
 /*
  * Assertions
@@ -264,7 +270,11 @@ extern void mock_assert(const int result, const char* const expression,
 /*%
  * Alignment
  */
-#define ISC_ALIGN(x, a) (((x) + (a) - 1) & ~((typeof(x))(a)-1))
+#ifdef __GNUC__
+#define ISC_ALIGN(x, a) (((x) + (a) - 1) & ~((typeof(x))(a) - 1))
+#else
+#define ISC_ALIGN(x, a) (((x) + (a) - 1) & ~((uintmax_t)(a) - 1))
+#endif
 
 /*%
  * Misc
