@@ -3,7 +3,7 @@
  *
  * This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
- * file, you can obtain one at https://mozilla.org/MPL/2.0/.
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
  * See the COPYRIGHT file distributed with this work for additional
  * information regarding copyright ownership.
@@ -17,35 +17,20 @@
 #include <isc/mutexblock.h>
 #include <isc/util.h>
 
-isc_result_t
+void
 isc_mutexblock_init(isc_mutex_t *block, unsigned int count) {
-	isc_result_t result;
 	unsigned int i;
 
 	for (i = 0; i < count; i++) {
-		result = isc_mutex_init(&block[i]);
-		if (result != ISC_R_SUCCESS) {
-			while (i > 0U) {
-				i--;
-				DESTROYLOCK(&block[i]);
-			}
-			return (result);
-		}
+		isc_mutex_init(&block[i]);
 	}
-
-	return (ISC_R_SUCCESS);
 }
 
-isc_result_t
+void
 isc_mutexblock_destroy(isc_mutex_t *block, unsigned int count) {
-	isc_result_t result;
 	unsigned int i;
 
 	for (i = 0; i < count; i++) {
-		result = isc_mutex_destroy(&block[i]);
-		if (result != ISC_R_SUCCESS)
-			return (result);
+		isc_mutex_destroy(&block[i]);
 	}
-
-	return (ISC_R_SUCCESS);
 }
